/****************************************************************************
 *                                                                          *
 * QNX 4.x TERM_* function compatibility layer for Curses                   *
 * Started by Frederick (Rick) A Niles <fniles@mitre.org>, March 2003       *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <ctype.h>
#include <sys/select.h>
#include <sys/qnxterm.h>

static WINDOW *major_win;
static int curses_on;
struct term_state_struct term_state;

void
term_type (int y, int x, void *string, int length, int attr)
{
  wattron (major_win, COLOR_PAIR (TERM_YELLOW | TERM_BLUE_BG));
  wattron (major_win, attr & 0xffffff00);
  wmove (major_win, y, x);
  if (length == 1) {
    chtype ch;
    char *s = (char *) string;

    if ((s[1] == 0) && (isdigit((int) s[0])))
      ch = s[0];
    else
      ch = *(chtype *) string;

    waddch (major_win, ch);
  }
  else
    wprintw (major_win, (char *) string);
  wattroff (major_win, attr);
}

void
term_color (short color_pair)
{
  wattron (major_win, COLOR_PAIR (color_pair));
}

void
term_printf (int y, int x, int attr, char *fmt, ...)
{
  va_list ap;
  wattron (major_win, attr);
  va_start (ap, fmt);
  wmove (major_win, y, x);
  vwprintw (major_win, fmt, ap);
  va_end (ap);
  wattroff (major_win, attr);
}

void
term_box_fill (int y, int x, int cols, int rows, int attr, int frame, int c)
{
  int i, j;
  wattron (major_win, (attr & 0xffffff00) | COLOR_PAIR (attr));
  for (i = 0; i < cols; i++)
    for (j = 0; j < rows; j++)
      mvwaddch (major_win, y + j, x + i, c);
  if (frame) {
    for (i = 0; i < cols; i++) {
      mvwaddch (major_win, y, x + i, ACS_HLINE);
      mvwaddch (major_win, y+rows-1, x + i, ACS_HLINE);
    }
    for (j = 0; j < rows; j++) {
      mvwaddch (major_win, y + j, x, ACS_VLINE);
      mvwaddch (major_win, y + j, x + cols-1, ACS_VLINE);
    }
    mvwaddch (major_win, y, x, ACS_ULCORNER);
    mvwaddch (major_win, y, x + cols-1, ACS_URCORNER);
    mvwaddch (major_win, y + rows-1, x, ACS_LLCORNER);
    mvwaddch (major_win, y + rows-1, x + cols-1, ACS_LRCORNER);
  }
  wattroff (major_win, attr); 
}

void
term_flush (void)
{
  if (curses_on) {
    touchwin (stdscr);
    refresh ();
  }
}

void
term_box_on (void)
{
}
void
term_box_off (void)
{
}
void
term_home (void)
{
  move (0, 0);
  endwin();
  curses_on = 0;
}
void
term_axis (int y, int x, int type, int style UNUSED,
           int inside_ticks, int outside_ticks, 
	   int length, int term UNUSED)
{
  int i, sym=0;

  wmove (major_win, y, x);
  switch (type) {
  case TERM_VERT_R:
    waddch (major_win, ACS_LRCORNER);
    for (i = 1; i < length-1; i++) {
      switch (2*(2*(i-1) % inside_ticks == 0) +
	      (2*(i-1) % outside_ticks == 0)) {
      case 0:
	sym = ACS_VLINE;
	break;
      case 1:
	sym = ACS_RTEE;
	break;
      case 2:
	sym = ACS_LTEE;
	break;
      case 3:
	sym = ACS_PLUS;
	break;
      }
      mvwaddch (major_win, y-i, x, sym);
    }
    mvwaddch (major_win, y-length+1, x, ACS_URCORNER);
    break;
  case TERM_VERT_L:
    waddch (major_win, ACS_LLCORNER);
    for (i = 1; i < length-1; i++) {
      switch (2*(2*(i-1) % inside_ticks == 0) +
	      (2*(i-1) % outside_ticks == 0)) {
      case 0:
	sym = ACS_VLINE;
	break;
      case 1:
	sym = ACS_LTEE;
	break;
      case 2:
	sym = ACS_RTEE;
	break;
      case 3:
	sym = ACS_PLUS;
	break;
      }
      mvwaddch (major_win, y-i, x, sym);
    }
    mvwaddch (major_win, y-length+1, x, ACS_ULCORNER);
    break;
  default:
    mvwprintw (major_win, 3, 3, "Error: term_axis(): type not implemented");
    break;
  }


}
void
term_bar (int y, int x, int type, 
	  int active, int length, int opt UNUSED)
{
  int i;

  if (type != TERM_BAR_VERT) {
    mvwprintw (major_win, 3, 3, 
	      "Error: term_bar() only vertical bar implemented");
    return;
  }

  wattron (major_win, COLOR_PAIR ((TERM_YELLOW | TERM_BLUE_BG)) | A_BOLD);

  for (i = 0; i < active; i++)
    mvwaddch (major_win, y-i, x, ACS_BLOCK);

  for (i = active; i < length; i++)
    mvwaddch (major_win, y-i, x, '.');

  wattroff (major_win, A_BOLD);
  term_color (TERM_RED | TERM_BLUE_BG);
}
void
term_clear (int i UNUSED)
{
  clear ();
}

void
term_fill (int i UNUSED)
{
}

int
term_load (void)
{
  /* Initialize the terminal (Curses) */
  if (initscr () == NULL)
    {
      printf ("initscr: Unable to initialize curses\n");
      exit (EXIT_FAILURE);
    }

  /* Standard goody settings for Curses */
  cbreak ();
  noecho ();
  nonl ();
  intrflush (stdscr, FALSE);
  keypad (stdscr, TRUE);

  /* no blocking input */
  nodelay (stdscr, TRUE);

  /* Invisible cursor */
  curs_set (0);

  term_state.num_rows = LINES;
  term_state.num_cols = COLS;
  term_state.box_arrow_up    = ACS_UARROW;
  term_state.box_arrow_down  = ACS_DARROW;
  term_state.box_arrow_right = ACS_RARROW;
  term_state.box_arrow_left  = ACS_LARROW;
  term_state.box_horizontal  = ACS_HLINE;
  term_state.box_vertical    = ACS_VLINE;
  term_state.box_diamond     = ACS_DIAMOND;
  term_state.box_degree      = ACS_DEGREE;


  if (start_color () == ERR) {
    endwin ();
    fprintf (stderr, "\nTerminal lacks color capability...\n");
    /* exit (EXIT_FAILURE); */
  }

  init_pair (TERM_RED     | TERM_BLUE_BG, COLOR_RED,    COLOR_BLUE);
  init_pair (TERM_CYAN    | TERM_BLUE_BG, COLOR_CYAN,   COLOR_BLUE);
  init_pair (TERM_YELLOW  | TERM_BLUE_BG, COLOR_YELLOW, COLOR_BLUE);
  init_pair (TERM_GREEN   | TERM_BLUE_BG, COLOR_GREEN,  COLOR_BLUE);
  init_pair (TERM_YELLOW  | TERM_CYAN_BG, COLOR_YELLOW, COLOR_CYAN);
  init_pair (TERM_CYAN    | TERM_BLACK_BG, COLOR_CYAN,  COLOR_BLACK);
  term_color (TERM_YELLOW | TERM_BLUE_BG);

  major_win = stdscr;
  wattron (major_win, COLOR_PAIR (TERM_CYAN | TERM_BLUE_BG));

  curses_on = 1;

  return 0;
}

void term_video_on(void) {}
void term_key(void) {}
void 
term_restore(void) 
{
  move(0, 0);
  clear();
  refresh();
}

int
kbhit (void)
{
  fd_set readfds;
  struct timeval timeout={0,0};
  FD_ZERO(&readfds);
  FD_SET(0, &readfds);
  return (select (1, &readfds, NULL, NULL, &timeout) == 1);
}
